﻿unit LorenzSystem;

interface

uses Graphics, Math, RejbrandCommon;

procedure Orbit(const σ, ρ, β: TASR; const P0: TASRVector; var AOrbit: TASSet; Coloured: boolean = false; SkipInitial: boolean = false); overload;
procedure Orbit(const P0: TASRVector; var AOrbit: TASSet; Coloured: boolean = false; SkipInitial: boolean = false); overload;

const

  VMAX = 500;

implementation

procedure Orbit(const σ, ρ, β: TASR; const P0: TASRVector; var AOrbit: TASSet; Coloured: boolean = false; SkipInitial: boolean = false);
const
  TMAX = 100;
  DT = 0.001;
var
  t: TASR;
  p, v: TASRVector;
  Colour: TColor;
begin
  t := 0;
  p := P0;
  AOrbit.ZeroMemory;
  AOrbit.BeginRapidRVect(ceil(TMAX/DT)+1);
  while t < TMAX do
  begin
    v := ASR3(σ * (p[1] - p[0]),
              p[0] * (ρ - p[2]) - p[1],
              p[0] * p[1] - β*p[2]);
    p := VectAdd(p, VectScale(v, DT));
    t := t + DT;
    if Coloured then
      Colour := GetPascalColor(HSVToRGB(HSV(Max(240 - 240 * VectNorm(v) / VMAX, 0), 1, 1)))
    else
      Colour := $00FFFFFF;
    if (not SkipInitial) or (t > 1) then
      AOrbit.RapidAddElement(ASR4(p[0], p[1], p[2], Colour));
  end;
  AOrbit.EndRapidRVect;
end;

procedure Orbit(const P0: TASRVector; var AOrbit: TASSet; Coloured: boolean = false; SkipInitial: boolean = false);
begin
  Orbit(10, 28, 8/3, P0, AOrbit, Coloured, SkipInitial);
end;


end.
